@@ -11,27 +11,65 @@ import time
 
 import re
 from devpi_common.vendor._pip import HTMLPage
-
 from devpi_common.url import URL
 from devpi_common.metadata import BasenameMeta
 from devpi_common.metadata import is_archive_of_project
 from devpi_common.validation import normalize_name
 from functools import partial
+from html.parser import HTMLParser
 from .config import hookimpl
 from .model import BaseStageCustomizer
 from .model import BaseStage, make_key_and_href, SimplelinkMeta
 from .model import ensure_boolean
-from .model import join_requires
+from .model import join_links_data
 from .readonly import ensure_deeply_readonly
 from .log import threadlog
+from .views import make_uuid_headers
 
 
 class Link(URL):
     def __init__(self, url="", *args, **kwargs):
         self.requires_python = kwargs.pop('requires_python', None)
+        self.yanked = kwargs.pop('yanked', None)
         URL.__init__(self, url, *args, **kwargs)
 
 
+class ProjectParser(HTMLParser):
+    def __init__(self, url):
+        HTMLParser.__init__(self)
+        self.projects = set()
+        self.baseurl = URL(url)
+        self.basehost = self.baseurl.replace(path='')
+        self.project = None
+
+    def handle_starttag(self, tag, attrs):
+        if tag == 'a':
+            self.project = None
+            attrs = dict(attrs)
+            if 'href' not in attrs:
+                return
+            href = attrs['href']
+            if '://' not in href:
+                project = href.rstrip('/').rsplit('/', 1)[-1]
+            else:
+                newurl = self.baseurl.joinpath(href)
+                # remove trailing slashes, so basename works correctly
+                newurl = newurl.asfile()
+                if not newurl.is_valid_http_url():
+                    return
+                if not newurl.path.startswith(self.baseurl.path):
+                    return
+                if self.basehost != newurl.replace(path=''):
+                    return
+                project = newurl.basename
+            self.project = project
+
+    def handle_endtag(self, tag):
+        if tag == 'a' and self.project:
+            self.projects.add(self.project)
+            self.project = None
+
+
 class IndexParser:
 
     def __init__(self, project):
@@ -58,16 +96,14 @@ class IndexParser:
 
     @property
     def releaselinks(self):
-        """ return sorted releaselinks list """
-        l = sorted(map(BasenameMeta, self.basename2link.values()),
-                   reverse=True)
+        l = map(BasenameMeta, self.basename2link.values())
         return [x.obj for x in l]
 
     def parse_index(self, disturl, html):
         p = HTMLPage(html, disturl.url)
         seen = set()
         for link in p.links:
-            newurl = Link(link.url, requires_python=link.requires_python)
+            newurl = Link(link.url, requires_python=link.requires_python, yanked=link.yanked)
             if not newurl.is_valid_http_url():
                 continue
             if is_archive_of_project(newurl, self.project):
@@ -92,7 +128,6 @@ class PyPIStage(BaseStage):
     def __init__(self, xom, username, index, ixconfig, customizer_cls):
         super(PyPIStage, self).__init__(
             xom, username, index, ixconfig, customizer_cls)
-        self.httpget = self.xom.httpget  # XXX is requests/httpget multi-thread safe?
         self.xom = xom
         self.offline = self.xom.config.offline_mode
         self.timeout = xom.config.request_timeout
@@ -101,6 +136,19 @@ class PyPIStage(BaseStage):
         # used to log about stale projects only once
         self._offline_logging = set()
 
+    def httpget(self, url, allow_redirects, timeout=None, extra_headers=None):
+        if self.xom.is_replica():
+            if extra_headers is None:
+                extra_headers = {}
+            (uuid, master_uuid) = make_uuid_headers(self.xom.config.nodeinfo)
+            rt = self.xom.replica_thread
+            token = rt.auth_serializer.dumps(uuid)
+            extra_headers[rt.H_REPLICA_UUID] = uuid
+            extra_headers[str('Authorization')] = 'Bearer %s' % token
+        return self.xom.httpget(
+            url=url, allow_redirects=allow_redirects, timeout=timeout,
+            extra_headers=extra_headers)
+
     @property
     def cache_expiry(self):
         return self.ixconfig.get(
@@ -249,24 +297,12 @@ class PyPIStage(BaseStage):
             self.mirror_url, allow_redirects=True, extra_headers=headers,
             timeout=timeout)
         if response.status_code != 200:
-            raise self.UpstreamError("URL %r returned %s %s",
+            raise self.UpstreamError(
+                "URL %r returned %s %s",
                 self.mirror_url, response.status_code, response.reason)
-        page = HTMLPage(response.text, response.url)
-        projects = set()
-        baseurl = URL(response.url)
-        basehost = baseurl.replace(path='')
-        for link in page.links:
-            newurl = URL(link.url)
-            # remove trailing slashes, so basename works correctly
-            newurl = newurl.asfile()
-            if not newurl.is_valid_http_url():
-                continue
-            if not newurl.path.startswith(baseurl.path):
-                continue
-            if basehost != newurl.replace(path=''):
-                continue
-            projects.add(newurl.basename)
-        return projects
+        parser = ProjectParser(response.url)
+        parser.feed(response.text)
+        return parser.projects
 
     def list_projects_perstage(self):
         """ return set of all projects served through the mirror. """
@@ -308,12 +344,14 @@ class PyPIStage(BaseStage):
         """ return True if we have some cached simpelinks information. """
         return self.key_projsimplelinks(project).exists()
 
-    def _save_cache_links(self, project, links, requires_python, serial):
+    def _save_cache_links(self, project, links, requires_python, yanked, serial):
         assert links != ()  # we don't store the old "Not Found" marker anymore
         assert isinstance(serial, int)
         assert project == normalize_name(project), project
-        data = {"serial": serial, "links": links,
-            "requires_python": requires_python}
+        data = {
+            "serial": serial, "links": links,
+            "requires_python": requires_python,
+            "yanked": yanked}
         key = self.key_projsimplelinks(project)
         old = key.get()
         if old != data:
@@ -330,19 +368,21 @@ class PyPIStage(BaseStage):
         self.cache_retrieve_times.refresh(project)
 
     def _load_cache_links(self, project):
-        is_expired, links_with_require_python, serial = True, None, -1
+        is_expired, links_with_data, serial = True, None, -1
 
         cache = self.key_projsimplelinks(project).get()
         if cache:
             is_expired = self.cache_retrieve_times.is_expired(project, self.cache_expiry)
             serial = cache["serial"]
-            links_with_require_python = join_requires(
-                cache["links"], cache.get("requires_python", []))
-            if self.offline and links_with_require_python:
-                links_with_require_python = ensure_deeply_readonly(list(
-                    filter(self._is_file_cached, links_with_require_python)))
+            links_with_data = join_links_data(
+                cache["links"],
+                cache.get("requires_python", []),
+                cache.get("yanked", []))
+            if self.offline and links_with_data:
+                links_with_data = ensure_deeply_readonly(list(
+                    filter(self._is_file_cached, links_with_data)))
 
-        return is_expired, links_with_require_python, serial
+        return is_expired, links_with_data, serial
 
     def _entry_from_href(self, href):
         # extract relpath from href by cutting of the hash
@@ -453,12 +493,13 @@ class PyPIStage(BaseStage):
             entries = [maplink(link) for link in releaselinks]
             links = [make_key_and_href(entry) for entry in entries]
             requires_python = [link.requires_python for link in releaselinks]
-            self._save_cache_links(project, links, requires_python, serial)
+            yanked = [link.yanked for link in releaselinks]
+            self._save_cache_links(project, links, requires_python, yanked, serial)
             # make project appear in projects list even
             # before we next check up the full list with remote
             threadlog.info("setting projects cache for %r", project)
             self.cache_projectnames.get_inplace().add(project)
-            return join_requires(links, requires_python)
+            return join_links_data(links, requires_python, yanked)
 
         try:
             return map_and_dump()
@@ -496,6 +537,8 @@ class PyPIStage(BaseStage):
             return map_and_dump()
 
     def has_project_perstage(self, project):
+        if self.is_project_cached(project):
+            return True
         try:
             self.get_simplelinks_perstage(project)
         except self.UpstreamNotFoundError:
@@ -533,6 +576,8 @@ class PyPIStage(BaseStage):
                     verdata['version'] = version
                 if sm.require_python is not None:
                     verdata['requires_python'] = sm.require_python
+                if sm.yanked:
+                    verdata['yanked'] = sm.yanked
                 elinks = verdata.setdefault("+elinks", [])
                 entrypath = sm._url.path
                 elinks.append({"rel": "releasefile", "entrypath": entrypath})
