@@ -12,7 +12,7 @@ import requests
 import socket
 import sys
 import time
-from .reqmock import reqmock  # noqa
+from .reqmock import reqmock, patch_reqsessionmock  # noqa
 from bs4 import BeautifulSoup
 from contextlib import closing
 from devpi_server import extpypi
@@ -47,32 +47,47 @@ def make_file_url(basename, content, stagename=None, baseurl="http://localhost/"
         s = s.format(stage=stagename)
     return s
 
-class TimeoutQueue(BaseQueue):
+
+class _TimeoutQueue(BaseQueue):
     def get(self, timeout=2):
         return BaseQueue.get(self, timeout=timeout)
 
 
 log = threadlog
 
+
 @pytest.fixture(autouse=True)
 def _clear():
     thread_clear_log()
 
 
-@pytest.yield_fixture
-def pool():
-    from devpi_server.mythread  import ThreadPool
-    pool = ThreadPool()
-    yield pool
-    pool.shutdown()
+LOWER_ARGON2_MEMORY_COST = 8
+LOWER_ARGON2_PARALLELISM = 1
+LOWER_ARGON2_TIME_COST = 1
+
+
+@pytest.fixture(autouse=True)
+def lower_argon2_parameters(monkeypatch):
+    from devpi_server.config import Config
+    import argon2
+
+    secret_parameters = argon2.Parameters(
+        type=argon2.low_level.Type.ID,
+        version=argon2.low_level.ARGON2_VERSION,
+        salt_len=16,
+        hash_len=16,
+        time_cost=LOWER_ARGON2_TIME_COST,
+        memory_cost=LOWER_ARGON2_MEMORY_COST,
+        parallelism=LOWER_ARGON2_PARALLELISM)
+
+    monkeypatch.setattr(
+        Config, "_secret_parameters", secret_parameters)
+
 
 @pytest.fixture
-def queue():
-    return TimeoutQueue()
+def TimeoutQueue():
+    return _TimeoutQueue
 
-@pytest.fixture
-def Queue():
-    return TimeoutQueue
 
 @pytest.fixture()
 def caplog(caplog):
@@ -95,9 +110,11 @@ def caplog(caplog):
     caplog.set_level(logging.NOTSET)
     return caplog
 
+
 @pytest.fixture
 def gentmp(request, tmpdir_factory):
     cache = []
+
     def gentmp(name=None):
         if not cache:
             prefix = re.sub(r"[\W]", "_", request.node.name)
@@ -109,14 +126,15 @@ def gentmp(request, tmpdir_factory):
             return basedir.mkdir(name)
         return py.path.local.make_numbered_dir(prefix="gentmp",
                 keep=0, rootdir=basedir, lock_timeout=None)
+
     return gentmp
 
 
-@pytest.yield_fixture(autouse=True)
+@pytest.fixture(autouse=True)
 def auto_transact(request):
     names = request.fixturenames
     if ("xom" not in names and "keyfs" not in names) or (
-        request.node.get_closest_marker("notransaction")):
+            request.node.get_closest_marker("notransaction")):
         yield
         return
     keyfs = request.getfixturevalue("keyfs")
@@ -137,29 +155,33 @@ def xom(request, makexom):
     return xom
 
 
-@pytest.yield_fixture(autouse=True, scope="session")
+@pytest.fixture(autouse=True, scope="session")
 def speed_up_sqlite():
     from devpi_server.keyfs_sqlite import Storage
     old = Storage.ensure_tables_exist
+
     def make_unsynchronous(self, old=old):
         conn = old(self)
         with self.get_connection() as conn:
             conn._sqlconn.execute("PRAGMA synchronous=OFF")
         return
+
     Storage.ensure_tables_exist = make_unsynchronous
     yield
     Storage.ensure_tables_exist = old
 
 
-@pytest.yield_fixture(autouse=True, scope="session")
+@pytest.fixture(autouse=True, scope="session")
 def speed_up_sqlite_fs():
     from devpi_server.keyfs_sqlite_fs import Storage
     old = Storage.ensure_tables_exist
+
     def make_unsynchronous(self, old=old):
         conn = old(self)
         with self.get_connection() as conn:
             conn._sqlconn.execute("PRAGMA synchronous=OFF")
         return
+
     Storage.ensure_tables_exist = make_unsynchronous
     yield
     Storage.ensure_tables_exist = old
@@ -197,17 +219,26 @@ def makexom(request, gentmp, httpget, monkeypatch, storage_info):
         from devpi_server import auth_basic
         from devpi_server import auth_devpi
         from devpi_server import model
+        from devpi_server import replica
+        from devpi_server import view_auth
+        from devpi_server import views
         plugins = [
             plugin[0] if isinstance(plugin, tuple) else plugin
             for plugin in plugins]
-        for plugin in [auth_basic, auth_devpi, extpypi, model, storage_info["_test_plugin"]]:
+        default_plugins = [
+            auth_basic, auth_devpi, extpypi, model, replica, view_auth, views,
+            storage_info["_test_plugin"]]
+        for plugin in default_plugins:
             if plugin not in plugins:
                 plugins.append(plugin)
         pm = get_pluginmanager(load_entrypoints=False)
         for plugin in plugins:
             pm.register(plugin)
         serverdir = gentmp()
-        fullopts = ["devpi-server", "--serverdir", serverdir] + list(opts)
+        if "--serverdir" in opts:
+            fullopts = ["devpi-server"] + list(opts)
+        else:
+            fullopts = ["devpi-server", "--serverdir", serverdir] + list(opts)
         if request.node.get_closest_marker("with_replica_thread"):
             fullopts.append("--master=http://localhost")
         if not request.node.get_closest_marker("no_storage_option"):
@@ -236,12 +267,8 @@ def makexom(request, gentmp, httpget, monkeypatch, storage_info):
         from devpi_server.main import init_default_indexes
         if not xom.config.args.master_url:
             init_default_indexes(xom)
-        if request.node.get_closest_marker("with_replica_thread"):
-            from devpi_server.replica import ReplicaThread
-            rt = ReplicaThread(xom)
-            xom.replica_thread = rt
-            xom.thread_pool.register(rt)
-            xom.thread_pool.start_one(rt)
+        if xom.is_replica() and request.node.get_closest_marker("with_replica_thread"):
+            xom.thread_pool.start_one(xom.replica_thread)
         if request.node.get_closest_marker("start_threads"):
             xom.thread_pool.start()
         elif request.node.get_closest_marker("with_notifier"):
@@ -252,10 +279,10 @@ def makexom(request, gentmp, httpget, monkeypatch, storage_info):
 
 
 @pytest.fixture
-def replica_xom(request, makexom):
+def replica_xom(request, makexom, secretfile):
     from devpi_server.replica import register_key_subscribers
     master_url = "http://localhost:3111"
-    xom = makexom(["--master", master_url])
+    xom = makexom(["--master", master_url, "--secretfile", secretfile.strpath])
     register_key_subscribers(xom)
     return xom
 
@@ -278,6 +305,7 @@ def maketestapp(request):
         return mt
     return maketestapp
 
+
 @pytest.fixture
 def makemapp(request, maketestapp, makexom):
     def makemapp(testapp=None, options=()):
@@ -292,10 +320,12 @@ def makemapp(request, maketestapp, makexom):
 @pytest.fixture
 def httpget(pypiurls):
     from .simpypi import make_simple_pkg_info
+
     class MockHTTPGet:
         def __init__(self):
             self.url2response = {}
             self._md5 = hashlib.md5()
+            self.call_log = []
 
         def __call__(self, url, allow_redirects=False, extra_headers=None, **kw):
             class mockresponse:
@@ -317,6 +347,12 @@ def httpget(pypiurls):
                                                          xself.url)
             r = mockresponse(url)
             log.debug("returning %s", r)
+            self.call_log.append(dict(
+                url=url,
+                allow_redirects=allow_redirects,
+                extra_headers=extra_headers,
+                kw=kw,
+                response=r))
             return r
 
         def mockresponse(self, mockurl, **kw):
@@ -348,14 +384,12 @@ def httpget(pypiurls):
 
     return MockHTTPGet()
 
-@pytest.fixture
-def filestore(xom):
-    return xom.filestore
 
 @pytest.fixture
 def keyfs(xom):
     return xom.keyfs
 
+
 @pytest.fixture
 def model(xom):
     return xom.model
@@ -369,16 +403,17 @@ def pypistage(devpiserver_makepypistage, xom):
 def add_pypistage_mocks(monkeypatch, httpget):
     # add some mocking helpers
     PyPIStage.url2response = httpget.url2response
+
     def mock_simple(self, name, text=None, pypiserial=10000, **kw):
         self.cache_retrieve_times.expire(name)
-        return self.httpget.mock_simple(name,
-                 text=text, pypiserial=pypiserial, **kw)
+        return self.xom.httpget.mock_simple(
+            name, text=text, pypiserial=pypiserial, **kw)
     monkeypatch.setattr(PyPIStage, "mock_simple", mock_simple, raising=False)
 
     def mock_simple_projects(self, projectlist):
         t = "".join('<a href="%s">%s</a>\n' % (name, name) for name in projectlist)
         threadlog.debug("patching simple page with: %s" %(t))
-        self.httpget.mockresponse(self.mirror_url, code=200, text=t)
+        self.xom.httpget.mockresponse(self.mirror_url, code=200, text=t)
 
     monkeypatch.setattr(PyPIStage, "mock_simple_projects",
                         mock_simple_projects, raising=False)
@@ -388,16 +423,19 @@ def add_pypistage_mocks(monkeypatch, httpget):
                    "content-type": mimetypes.guess_type(path),
                    "last-modified": "today",}
         url = URL(self.mirror_url).joinpath(path)
-        return self.httpget.mockresponse(url.url, content=content,
-                                         headers=headers, **kw)
+        return self.xom.httpget.mockresponse(
+            url.url, content=content, headers=headers, **kw)
     monkeypatch.setattr(PyPIStage, "mock_extfile", mock_extfile, raising=False)
 
+
 @pytest.fixture
 def pypiurls():
     from devpi_server.main import _pypi_ixconfig_default
+
     class PyPIURL:
         def __init__(self):
             self.simple = _pypi_ixconfig_default['mirror_url']
+
     return PyPIURL()
 
 
@@ -562,7 +600,8 @@ class Mapp(MappMixin):
         assert r.status_code == code
         if code in (200,201):
             assert r.json["result"]["type"] == indexconfig.get("type", "stage")
-            return self.use("%s/%s" % (user, index))
+            if use:
+                return self.use("%s/%s" % (user, index))
         if code in (400,):
             return r.json["message"]
 
@@ -743,15 +782,17 @@ from webtest import TestApp as TApp
 from webtest import TestResponse
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def noiter(monkeypatch, request):
     l = []
+
     @property
     def body(self):
         if self.headers["Content-Type"] != "application/octet-stream":
             return self.body_old
         if self.app_iter:
             l.append(self.app_iter)
+
     monkeypatch.setattr(TestResponse, "body_old", TestResponse.body,
                         raising=False)
     monkeypatch.setattr(TestResponse, "body", body)
@@ -832,7 +873,6 @@ class MyTestApp(TApp):
     def get_json(self, *args, **kwargs):
         headers = kwargs.setdefault("headers", {})
         headers["Accept"] = "application/json"
-        self.x = 1
         return self.get(*args, **kwargs)
 
 
@@ -910,7 +950,7 @@ def wait_for_port(host, port, timeout=60):
         "The port %s on host %s didn't become accessible" % (port, host))
 
 
-@pytest.yield_fixture(scope="class")
+@pytest.fixture(scope="class")
 def server_directory():
     import tempfile
     srvdir = py.path.local(
@@ -968,19 +1008,29 @@ def master_serverdir(server_directory):
     return server_directory.join("master")
 
 
-@pytest.yield_fixture(scope="class")
-def master_host_port(request, call_devpi_in_dir, master_serverdir, server_directory, storage_info):
+@pytest.fixture(scope="class")
+def secretfile(server_directory):
     import base64
-    import os
+    import secrets
     secretfile = server_directory.join('testserver.secret')
     if not secretfile.exists():
-        secretfile.write(base64.b64encode(os.urandom(32)))
+        secretfile.write(base64.b64encode(secrets.token_bytes(32)))
+        if sys.platform != "win32":
+            secretfile.chmod(0o600)
+    return secretfile
+
+
+@pytest.fixture(scope="class")
+def master_host_port(request, call_devpi_in_dir, master_serverdir, server_directory, secretfile, storage_info):
     host = 'localhost'
     port = get_open_port(host)
     args = [
         "devpi-server",
         "--role", "master",
         "--secretfile", secretfile.strpath,
+        "--argon2-memory-cost", str(LOWER_ARGON2_MEMORY_COST),
+        "--argon2-parallelism", str(LOWER_ARGON2_PARALLELISM),
+        "--argon2-time-cost", str(LOWER_ARGON2_TIME_COST),
         "--host", host,
         "--port", str(port),
         "--requests-only"]
@@ -1011,12 +1061,16 @@ def replica_serverdir(server_directory):
     return server_directory.join("replica")
 
 
-@pytest.yield_fixture(scope="class")
-def replica_host_port(request, call_devpi_in_dir, master_host_port, replica_serverdir, storage_info):
+@pytest.fixture(scope="class")
+def replica_host_port(request, call_devpi_in_dir, master_host_port, replica_serverdir, secretfile, storage_info):
     host = 'localhost'
     port = get_open_port(host)
     args = [
         "devpi-server",
+        "--secretfile", secretfile.strpath,
+        "--argon2-memory-cost", str(LOWER_ARGON2_MEMORY_COST),
+        "--argon2-parallelism", str(LOWER_ARGON2_PARALLELISM),
+        "--argon2-time-cost", str(LOWER_ARGON2_TIME_COST),
         "--host", host, "--port", str(port)]
     storage_args = [
         "--serverdir", replica_serverdir.strpath]
@@ -1103,7 +1157,7 @@ def _nginx_host_port(host, port, call_devpi_in_dir, server_directory):
     return (p, nginx_port)
 
 
-@pytest.yield_fixture(scope="class")
+@pytest.fixture(scope="class")
 def nginx_host_port(request, call_devpi_in_dir, server_directory):
     if sys.platform.startswith("win"):
         pytest.skip("no nginx on windows")
@@ -1118,9 +1172,12 @@ def nginx_host_port(request, call_devpi_in_dir, server_directory):
         p.wait()
 
 
-@pytest.yield_fixture(scope="class")
-def nginx_replica_host_port(replica_host_port, call_devpi_in_dir, server_directory):
-    (host, port) = replica_host_port
+@pytest.fixture(scope="class")
+def nginx_replica_host_port(request, call_devpi_in_dir, server_directory):
+    if sys.platform.startswith("win"):
+        pytest.skip("no nginx on windows")
+    # we need the skip above before master_host_port is called
+    (host, port) = request.getfixturevalue("replica_host_port")
     (p, nginx_port) = _nginx_host_port(
         host, port, call_devpi_in_dir, server_directory)
     try:
@@ -1152,7 +1209,7 @@ def simpypi(simpypiserver):
     return simpypiserver.simpypi
 
 
-### incremental testing
+# incremental testing
 
 def pytest_runtest_makereport(item, call):
     if "incremental" in item.keywords:
@@ -1160,6 +1217,7 @@ def pytest_runtest_makereport(item, call):
             parent = item.parent
             parent._previousfailed = item
 
+
 def pytest_runtest_setup(item):
     if "incremental" in item.keywords:
         previousfailed = getattr(item.parent, "_previousfailed", None)
@@ -1171,6 +1229,7 @@ def pytest_runtest_setup(item):
 def gen():
     return Gen()
 
+
 class Gen:
     def __init__(self):
         self._md5 = hashlib.md5()
@@ -1185,7 +1244,7 @@ class Gen:
         return URL(link)
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def pyramidconfig():
     from pyramid.testing import setUp, tearDown
     config = setUp()
@@ -1193,7 +1252,7 @@ def pyramidconfig():
     tearDown()
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def dummyrequest(pyramidconfig):
     from pyramid.testing import DummyRequest
     request = DummyRequest()
@@ -1204,8 +1263,10 @@ def dummyrequest(pyramidconfig):
 @pytest.fixture
 def blank_request():
     from pyramid.request import Request
+
     def blank_request(*args, **kwargs):
         return Request.blank("/blankpath", *args, **kwargs)
+
     return blank_request
 
 
