@@ -50,37 +50,41 @@ class TestParser:
         p = sub.add_parser("hello")
         assert isinstance(p, MyArgumentParser)
 
+
 class TestConfig:
     def test_parse_secret(self, caplog, tmpdir):
         # create a secret file
-        p = tmpdir.join("secret")
-        secret = "qwoieuqwelkj123"
+        configdir = tmpdir.ensure_dir('config')
+        configdir.chmod(0o700)
+        p = configdir.join("secret")
+        secret = b"qwoieuqwelkj1234qwoieuqwelkj1234"
         p.write(secret)
+        p.chmod(0o600)
         # and use it
         caplog.clear()
         config = make_config(["devpi-server", "--secretfile=%s" % p])
         assert config.args.secretfile == str(p)
         assert config.secretfile == str(p)
-        assert config.secret == secret
+        assert config.basesecret == secret
         recs = caplog.getrecords(".*new random secret.*")
         assert len(recs) == 0
         # now check the default
         caplog.clear()
-        config = make_config(["devpi-server", "--serverdir", tmpdir.strpath])
+        config = make_config(["devpi-server", "--serverdir", configdir.strpath])
         assert config.args.secretfile is None
         assert config.secretfile is None
-        assert config.secret != secret
+        assert config.basesecret != secret
         recs = caplog.getrecords(".*new random secret.*")
         assert len(recs) == 1
         # remember the secret
-        prev_secret = config.secret
+        prev_secret = config.basesecret
         # each startup without a secret file creates a new random secret
         caplog.clear()
-        config = make_config(["devpi-server", "--serverdir", tmpdir.strpath])
+        config = make_config(["devpi-server", "--serverdir", configdir.strpath])
         assert config.args.secretfile is None
         assert config.secretfile is None
-        assert config.secret != secret
-        assert config.secret != prev_secret
+        assert config.basesecret != secret
+        assert config.basesecret != prev_secret
         recs = caplog.getrecords(".*new random secret.*")
         assert len(recs) == 1
 
@@ -88,21 +92,68 @@ class TestConfig:
         import warnings
         warnings.simplefilter("always")
         # setup secret file in old default location
-        p = tmpdir.join(".secret")
-        secret = "qwoieuqwelkj123"
+        configdir = tmpdir.ensure_dir('config')
+        configdir.chmod(0o700)
+        p = configdir.join(".secret")
+        secret = b"qwoieuqwelkj1234qwoieuqwelkj1234"
         p.write(secret)
+        p.chmod(0o600)
         caplog.clear()
-        config = make_config(["devpi-server", "--serverdir", tmpdir.strpath])
+        config = make_config(["devpi-server", "--serverdir", configdir.strpath])
         # the existing file should be used
         assert config.args.secretfile is None
         assert config.secretfile == str(p)
-        assert config.secret == secret
+        assert config.basesecret == secret
         recs = caplog.getrecords(".*new random secret.*")
         assert len(recs) == 0
         assert len(recwarn) == 1
         warning = recwarn.pop(Warning)
         assert 'deprecated existing secret' in warning.message.args[0]
 
+    def test_secret_complexity(self, tmpdir):
+        # create a secret file with too short secret
+        configdir = tmpdir.ensure_dir('config')
+        configdir.chmod(0o700)
+        p = configdir.join("secret")
+        secret = b"qwoieuqwelkj123"
+        p.write(secret)
+        p.chmod(0o600)
+        # and use it
+        config = make_config(["devpi-server", "--secretfile=%s" % p])
+        with pytest.raises(Fatal, match="at least 32 characters"):
+            config.basesecret
+        # create a secret file which is too repetitive
+        p = configdir.join("secret")
+        secret = b"12345" * 7
+        p.write(secret)
+        p.chmod(0o600)
+        # and use it
+        config = make_config(["devpi-server", "--secretfile=%s" % p])
+        with pytest.raises(Fatal, match="less repetition"):
+            config.basesecret
+
+    @pytest.mark.skipif("sys.platform == 'win32'")
+    def test_secretfile_permissions(self, tmpdir):
+        # create a secret file with too short secret
+        configdir = tmpdir.ensure_dir('config')
+        configdir.chmod(0o777)
+        p = configdir.join("secret")
+        secret = b"qwoieuqwelkj123"
+        p.write(secret)
+        p.chmod(0o677)
+        config = make_config(["devpi-server", "--secretfile=%s" % p])
+        with pytest.raises(Fatal, match="file is world accessible"):
+            config.basesecret
+        p.chmod(0o670)
+        with pytest.raises(Fatal, match="file is group accessible"):
+            config.basesecret
+        p.chmod(0o600)
+        with pytest.raises(Fatal, match="folder of the given secret file is group writable"):
+            config.basesecret
+        configdir.chmod(0o707)
+        with pytest.raises(Fatal, match="folder of the given secret file is world writable"):
+            config.basesecret
+
     def test_devpi_serverdir_env(self, tmpdir, monkeypatch):
         monkeypatch.setenv("DEVPI_SERVERDIR", tmpdir.strpath)
         config = make_config(["devpi-server"])
@@ -167,9 +218,8 @@ class TestConfig:
     def test_replica_role_missing_master_url(self, tmpdir):
         config = make_config(["devpi-server", "--role=replica",
                              "--serverdir", str(tmpdir)])
-        with pytest.raises(Fatal) as excinfo:
+        with pytest.raises(Fatal, match="need to specify --master-url"):
             config.init_nodeinfo()
-        assert "need to specify --master-url" in str(excinfo.value)
 
     def test_uuid(self, tmpdir):
         config = make_config(["devpi-server", "--serverdir", str(tmpdir)])
@@ -187,10 +237,12 @@ class TestConfig:
 
     def test_add_parser_options_called(self):
         l = []
+
         class Plugin:
             @hookimpl
             def devpiserver_add_parser_options(self, parser):
                 l.append(parser)
+
         pm = get_pluginmanager()
         pm.register(Plugin())
         parseoptions(pm, ["devpi-server"])
@@ -238,6 +290,47 @@ class TestConfig:
         makexom(plugins=(plugin,), opts=options)
         assert plugin.settings == {"bar": "ham"}
 
+    @pytest.mark.parametrize('opts, expected', [
+        # defaults
+        ((), dict(host='localhost', port=3141)),
+        # host/port
+        (('--host', 'foo'), dict(host='foo', port=3141)),
+        (('--port', '1234'), dict(host='localhost', port=1234)),
+        (('--host', 'foo', '--port', '1234'), dict(host='foo', port=1234)),
+        # listen
+        (('--listen', '*:3141'), dict(listen='*:3141')),
+        (('--listen', '127.0.0.1:3141'), dict(listen='127.0.0.1:3141')),
+        (('--listen', '[::1]:3141'), dict(listen='[::1]:3141')),
+        (('--listen', '127.0.0.1:3141', '--listen', '[::1]:3142'), dict(listen='127.0.0.1:3141 [::1]:3142')),
+        (('--host', 'foo', '--listen', '127.0.0.1:3141'), Fatal('You can use either --listen or --host/--port, not both together.')),
+        (('--port', '1234', '--listen', '127.0.0.1:3141'), Fatal('You can use either --listen or --host/--port, not both together.')),
+        (('--host', 'foo', '--port', '1234', '--listen', '127.0.0.1:3141'), Fatal('You can use either --listen or --host/--port, not both together.')),
+    ])
+    def test_waitress_info_listen_host_port(self, expected, opts):
+        config = make_config(("devpi-server",) + opts)
+        try:
+            result = {
+                k: v
+                for k, v in config.waitress_info['kwargs'].items()
+                if k in ('host', 'port', 'listen')}
+        except Exception as e:
+            if not isinstance(e, expected.__class__):
+                raise
+            assert e.args == expected.args
+        else:
+            assert result == expected
+
+    def test_waitress_info_trusted_proxy(self):
+        config = make_config((
+            "devpi-server",
+            "--trusted-proxy", "127.0.0.1",
+            "--trusted-proxy-count", "2",
+            "--trusted-proxy-headers", "x-forwarded-for x-forwarded-host x-forwarded-proto x-forwarded-port"))
+        kwargs = config.waitress_info['kwargs']
+        assert kwargs["trusted_proxy"] == "127.0.0.1"
+        assert kwargs["trusted_proxy_count"] == 2
+        assert kwargs["trusted_proxy_headers"] == "x-forwarded-for x-forwarded-host x-forwarded-proto x-forwarded-port"
+
 
 class TestConfigFile:
     @pytest.fixture(params=(True, False))
@@ -292,6 +385,18 @@ class TestConfigFile:
         (out, err) = capsys.readouterr()
         assert "argument --port: invalid int value: 'foo'" in err
 
+    def test_requests_only(self, make_yaml_config):
+        yaml_path = make_yaml_config(textwrap.dedent("""\
+            devpi-server:
+              requests-only: false"""))
+        config = make_config(["devpi-server", "-c", yaml_path])
+        assert config.args.requests_only is False
+        yaml_path = make_yaml_config(textwrap.dedent("""\
+            devpi-server:
+              requests-only: true"""))
+        config = make_config(["devpi-server", "-c", yaml_path])
+        assert config.args.requests_only is True
+
     @pytest.mark.no_storage_option
     def test_storage_backend_options(self, makexom, make_yaml_config):
         class Plugin:
