@@ -120,6 +120,37 @@ class TestIndexParsing:
         assert link.hash_spec == "md5=pony"
         assert link.requires_python is None
 
+    def test_parse_index_with_yanked(self):
+        result = parse_index(
+            self.simplepy,
+            """<a href="pkg/py-1.0.zip" data-yanked="" />""")
+        assert len(result.releaselinks) == 1
+        link, = result.releaselinks
+        assert link.basename == "py-1.0.zip"
+        assert link.yanked is True
+
+    def test_parse_index_with_yanked_hash_spec_is_better(self):
+        result = parse_index(self.simplepy,
+            """<a href="pkg/py-1.0.zip" data-yanked="" />
+               <a href="pkg/py-1.0.zip#md5=pony"/>
+        """)
+        assert len(result.releaselinks) == 1
+        link, = result.releaselinks
+        assert link.basename == "py-1.0.zip"
+        assert link.hash_spec == "md5=pony"
+        assert link.yanked is False
+
+    def test_parse_index_with_yanked_first_with_hash_spec_kept(self):
+        result = parse_index(self.simplepy,
+            """<a href="pkg/py-1.0.zip#md5=pony"/>
+               <a href="pkg/py-1.0.zip#md5=pony" data-yanked="" />
+        """)
+        assert len(result.releaselinks) == 1
+        link, = result.releaselinks
+        assert link.basename == "py-1.0.zip"
+        assert link.hash_spec == "md5=pony"
+        assert link.yanked is False
+
     @pytest.mark.parametrize("basename", [
         "py-1.3.1.tar.gz",
         "py-1.3.1-1.fc12.src.rpm",
@@ -148,7 +179,6 @@ class TestIndexParsing:
         result = parse_index(simple, '<a href="http:" />')
         assert len(result.releaselinks) == 0
 
-
     def test_parse_index_ftp_ignored_for_now(self):
         result = parse_index(self.simplepy,
             """<a href="http://bb.org/download/py-1.0.zip" />
@@ -193,7 +223,6 @@ class TestIndexParsing:
                   rel="download">whatever2</a> """)
         assert len(result.releaselinks) == 0
 
-
     def test_releasefile_md5_matching_and_ordering(self):
         """ check that md5-links win over non-md5 links anywhere.
         And otherwise the links from the index page win over scraped ones.
@@ -217,11 +246,13 @@ class TestIndexParsing:
         assert link2.url == "http://pylib.org/py-1.4.11.zip#md5=1111"
         assert link3.url == "https://pypi.org/pkg/py-1.4.10.zip#md5=2222"
 
+
 def test_get_updated(pypistage):
     c = pypistage.cache_retrieve_times
     c2 = pypistage.cache_retrieve_times
     return c == c2
 
+
 class TestExtPYPIDB:
     def test_parse_project_nomd5(self, pypistage):
         pypistage.mock_simple("pytest", pkgver="pytest-1.0.zip")
@@ -367,10 +398,11 @@ class TestExtPYPIDB:
         # will result in the request response to have a "real" URL of
         # https://pypi.org/simple/hello-world because of the
         # new pypi normalization code
-        pypistage.httpget.mock_simple("hello-world",
-                '<a href="Hello_World-1.0.tar.gz" /a>',
-                code=200,
-                url="https://pypi.org/simple/hello-world",)
+        pypistage.xom.httpget.mock_simple(
+            "hello-world",
+            '<a href="Hello_World-1.0.tar.gz" /a>',
+            code=200,
+            url="https://pypi.org/simple/hello-world",)
         l = pypistage.get_releaselinks("Hello_World")
         assert len(l) == 1
 
@@ -388,6 +420,15 @@ class TestExtPYPIDB:
         (link,) = pypistage.get_releaselinks("foo")
         assert link.require_python == '<3'
 
+    def test_yanked_caching(self, pypistage):
+        pypistage.mock_simple("foo", text='<a href="foo-1.0.tar.gz" data-yanked=""></a>')
+        (link,) = pypistage.get_releaselinks("foo")
+        assert link.yanked is True
+        # make sure we get the cached data, if not throw an error
+        pypistage.httpget = None
+        (link,) = pypistage.get_releaselinks("foo")
+        assert link.yanked is True
+
     @pytest.mark.nomocking
     @pytest.mark.notransaction
     def test_offline_requires_python(self, mapp, simpypi, testapp, xom):
@@ -413,11 +454,37 @@ class TestExtPYPIDB:
         assert link.text == 'pkg-0.5.zip'
         assert link.get('data-requires-python') == '<3'
 
+    @pytest.mark.nomocking
+    @pytest.mark.notransaction
+    def test_offline_yanked(self, mapp, simpypi, testapp, xom):
+        mapp.login('root')
+        mapp.modify_index("root/pypi", indexconfig=dict(type='mirror', mirror_url=simpypi.simpleurl))
+        # turn off offline mode for preparations
+        xom.config.args.offline_mode = False
+        content = b'13'
+        simpypi.add_release('pkg', pkgver='pkg-0.5.zip', yanked=True)
+        simpypi.add_release('pkg', pkgver='pkg-1.0.zip')
+        simpypi.add_file('/pkg/pkg-0.5.zip', content)
+        r = testapp.xget(200, '/root/pypi/+simple/pkg/')
+        (link0, link1) = getlinks(r.text)
+        assert link0.text == 'pkg-1.0.zip'
+        assert link0.get('data-yanked') is None
+        assert link1.text == 'pkg-0.5.zip'
+        assert link1.get('data-yanked') == ""
+        testapp.xget(200, link1['href'].replace('../../', '/root/pypi/'))
+        # turn on offline mode for test
+        xom.config.args.offline_mode = True
+        r = testapp.get('/root/pypi/+simple/pkg/')
+        (link,) = getlinks(r.text)
+        assert link.text == 'pkg-0.5.zip'
+        assert link.get('data-yanked') == ""
+
 
 @pytest.mark.nomockprojectsremote
 class TestPyPIStageprojects:
     def test_get_remote_projects(self, pypistage):
-        pypistage.httpget.mockresponse(pypistage.mirror_url, code=200, text="""
+        pypistage.xom.httpget.mockresponse(
+            pypistage.mirror_url, code=200, text="""
             <html><head><title>Simple Index</title>
             <meta name="api-version" value="2" /></head>
             <body>
@@ -430,8 +497,21 @@ class TestPyPIStageprojects:
         s = pypistage.list_projects_perstage()
         assert s == set(["ploy-ansible", "devpi-server", "django"])
 
+    def test_get_remote_projects_doctype(self, pypistage):
+        pypistage.xom.httpget.mockresponse(
+            pypistage.mirror_url, code=200, text="""
+            <!DOCTYPE html>
+            <html><head><title>Simple Index</title>
+            <meta name="api-version" value="2" /></head>
+            <body>
+                <a href='devpi-server'>devpi-server</a><br/>
+            </body></html>""")
+        x = pypistage._get_remote_projects()
+        assert x == set(["devpi-server"])
+
     def test_single_project_access_updates_projects(self, pypistage):
-        pypistage.httpget.mockresponse(pypistage.mirror_url, code=200, text="""
+        pypistage.xom.httpget.mockresponse(
+            pypistage.mirror_url, code=200, text="""
             <body>
                 <a href='django'>Django</a><br/>
             </body>""")
@@ -447,7 +527,8 @@ class TestPyPIStageprojects:
         assert pypistage.list_projects_perstage() == set(["proj1", "proj2", "django"])
 
     def test_name_cache_expiration_updated_when_no_names_changed(self, httpget, pypistage):
-        pypistage.httpget.mockresponse(pypistage.mirror_url, code=200, text="""
+        pypistage.xom.httpget.mockresponse(
+            pypistage.mirror_url, code=200, text="""
             <body>
                 <a href='django'>Django</a><br/>
             </body>""")
@@ -472,6 +553,7 @@ def raise_ValueError():
 @pytest.mark.nomocking
 def test_requests_httpget_negative_status_code(xom, monkeypatch):
     l = []
+
     def r(*a, **k):
         l.append(1)
         raise requests.exceptions.RequestException()
