@@ -10,6 +10,7 @@ from devpi_server.main import tween_request_profiling
 import devpi_server
 import os
 
+
 @pytest.fixture
 def ground_wsgi_run(monkeypatch):
     monkeypatch.setattr(mythread.ThreadPool, "live", lambda *args: 0 / 0)
@@ -32,12 +33,14 @@ def test_pkgresources_version_matches_init():
     ver = devpi_server.__version__
     assert pkg_resources.get_distribution("devpi_server").version == ver
 
+
 def test_version(capfd):
     main(["devpi-server", "--version"])
     out, err = capfd.readouterr()
     assert not err  # not logging output
     assert devpi_server.__version__ in out.strip()
 
+
 def test_check_compatible_version_earlier(config, monkeypatch):
     monkeypatch.setattr("devpi_server.main.get_state_version", lambda cfg: "1.0")
     with pytest.raises(Fatal):
@@ -79,10 +82,12 @@ def test_check_incompatible_version_raises(config):
 
 def test_pyramid_configure_called(makexom):
     l = []
+
     class Plugin:
         @hookimpl
         def devpiserver_pyramid_configure(self, config, pyramid_config):
             l.append((config, pyramid_config))
+
     xom = makexom(plugins=[Plugin()])
     xom.create_app()
     assert len(l) == 1
@@ -101,26 +106,22 @@ def test_requests_only(makexom):
 
 
 @wsgi_run_throws
-@pytest.mark.parametrize("use_option", (False, True))
-def test_run_commands_called(tmpdir, use_option):
+def test_run_commands_called(tmpdir):
     from devpi_server.init import init
     from devpi_server.main import _main, get_pluginmanager
     l = []
+
     class Plugin:
         @hookimpl
         def devpiserver_cmdline_run(self, xom):
             l.append(xom)
             return 1
+
     pm = get_pluginmanager()
     pm.register(Plugin())
-    if use_option:
-        result = _main(
-            argv=["devpi-server", "--init", "--serverdir", str(tmpdir)],
-            pluginmanager=pm)
-    else:
-        result = init(
-            argv=["devpi-init", "--serverdir", str(tmpdir)],
-            pluginmanager=pm)
+    result = init(
+        argv=["devpi-init", "--serverdir", str(tmpdir)],
+        pluginmanager=pm)
     result = _main(
         argv=["devpi-server", "--serverdir", str(tmpdir)],
         pluginmanager=pm)
@@ -130,8 +131,7 @@ def test_run_commands_called(tmpdir, use_option):
 
 
 @wsgi_run_throws
-@pytest.mark.parametrize("use_option", (False, True))
-def test_fatal_if_no_storage_and_no_sqlite_file(tmpdir, use_option):
+def test_fatal_if_no_storage_and_no_sqlite_file(tmpdir):
     from devpi_server.init import init
     from devpi_server.main import _main, get_pluginmanager
 
@@ -141,14 +141,9 @@ def test_fatal_if_no_storage_and_no_sqlite_file(tmpdir, use_option):
             return 1
     pm = get_pluginmanager()
     pm.register(Plugin())
-    if use_option:
-        _main(
-            argv=["devpi-server", "--init", "--serverdir", str(tmpdir)],
-            pluginmanager=pm)
-    else:
-        init(
-            argv=["devpi-init", "--serverdir", str(tmpdir)],
-            pluginmanager=pm)
+    init(
+        argv=["devpi-init", "--serverdir", str(tmpdir)],
+        pluginmanager=pm)
     _main(
         argv=["devpi-server", "--serverdir", str(tmpdir)],
         pluginmanager=pm)
@@ -162,25 +157,21 @@ def test_fatal_if_no_storage_and_no_sqlite_file(tmpdir, use_option):
 
 
 @wsgi_run_throws
-@pytest.mark.parametrize("use_option", (False, True))
-def test_main_starts_server_if_run_commands_returns_none(tmpdir, use_option):
+def test_main_starts_server_if_run_commands_returns_none(tmpdir):
     from devpi_server.init import init
     from devpi_server.main import _main, get_pluginmanager
     l = []
+
     class Plugin:
         @hookimpl
         def devpiserver_cmdline_run(self, xom):
             l.append(xom)
+
     pm = get_pluginmanager()
     pm.register(Plugin())
-    if use_option:
-        _main(
-            argv=["devpi-server", "--init", "--serverdir", str(tmpdir)],
-            pluginmanager=pm)
-    else:
-        init(
-            argv=["devpi-init", "--serverdir", str(tmpdir)],
-            pluginmanager=pm)
+    init(
+        argv=["devpi-init", "--serverdir", str(tmpdir)],
+        pluginmanager=pm)
     with pytest.raises(ZeroDivisionError):
         _main(
             argv=["devpi-server", "--serverdir", str(tmpdir)],
@@ -240,6 +231,7 @@ def test_offline_mode_httpget_returns_server_error(makexom, url, allowRedirect):
 @pytest.mark.nomocking
 def test_replica_max_retries_option(makexom, monkeypatch):
     from devpi_server.main import new_requests_session as orig_new_requests_session
+
     def new_requests_session(*args, **kwargs):
         _max_retries = None
         if 'max_retries' in kwargs:
@@ -274,16 +266,17 @@ def test_request_args_timeout_handover(makexom, input_set):
     xom.httpget("http://whatever", allow_redirects=False, timeout=input_set['kwarg'])
 
 
-def test_no_root_pypi_option(makexom):
-    xom = makexom(["--no-root-pypi"])
+def test_no_root_pypi_option(gentmp, makexom, storage_info):
+    from devpi_server.init import init
+    serverdir = gentmp()
+    argv = ["devpi-init", "--serverdir", serverdir, "--no-root-pypi"]
+    if storage_info["name"] != "sqlite":
+        argv.append("--storage=%s" % storage_info["name"])
+    init(argv=argv)
+    xom = makexom(opts=("--serverdir", serverdir))
     with xom.keyfs.transaction(write=False):
         stage = xom.model.getstage('root/pypi')
         assert stage is None
-    xom = makexom()
-    with xom.keyfs.transaction(write=False):
-        stage = xom.model.getstage('root/pypi')
-        assert stage is not None
-        assert stage.name == 'root/pypi'
 
 
 def test_no_init_empty_directory(call_devpi_in_dir, tmpdir):
@@ -293,15 +286,11 @@ def test_no_init_empty_directory(call_devpi_in_dir, tmpdir):
     result.stderr.fnmatch_lines("*contains no devpi-server data*")
 
 
-@pytest.mark.parametrize("use_option", (False, True))
-def test_init_empty_directory(call_devpi_in_dir, monkeypatch, tmpdir, use_option):
+def test_init_empty_directory(call_devpi_in_dir, monkeypatch, tmpdir):
     monkeypatch.setattr("devpi_server.config.Config.init_nodeinfo", lambda x: 0/0)
     assert not len(os.listdir(tmpdir.strpath))
     with pytest.raises(ZeroDivisionError):
-        if use_option:
-            call_devpi_in_dir(tmpdir, ["devpi-server", "--init"])
-        else:
-            call_devpi_in_dir(tmpdir, ["devpi-init"])
+        call_devpi_in_dir(tmpdir, ["devpi-init"])
 
 
 def test_no_init_no_server_directory(call_devpi_in_dir, tmpdir):
@@ -312,26 +301,18 @@ def test_no_init_no_server_directory(call_devpi_in_dir, tmpdir):
     result.stderr.fnmatch_lines("*contains no devpi-server data*")
 
 
-@pytest.mark.parametrize("use_option", (False, True))
-def test_init_no_server_directory(call_devpi_in_dir, monkeypatch, tmpdir, use_option):
+def test_init_no_server_directory(call_devpi_in_dir, monkeypatch, tmpdir):
     monkeypatch.setattr("devpi_server.config.Config.init_nodeinfo", lambda x: 0/0)
     tmpdir.ensure("foo")
     assert os.listdir(tmpdir.strpath) == ["foo"]
     with pytest.raises(ZeroDivisionError):
-        if use_option:
-            call_devpi_in_dir(tmpdir, ["devpi-server", "--init"])
-        else:
-            call_devpi_in_dir(tmpdir, ["devpi-init"])
+        call_devpi_in_dir(tmpdir, ["devpi-init"])
 
 
-@pytest.mark.parametrize("use_option", (False, True))
-def test_init_server_directory(call_devpi_in_dir, tmpdir, use_option):
+def test_init_server_directory(call_devpi_in_dir, tmpdir):
     tmpdir.ensure(".nodeinfo")
     assert os.listdir(tmpdir.strpath) == [".nodeinfo"]
-    if use_option:
-        result = call_devpi_in_dir(tmpdir, ["devpi-server", "--init"])
-    else:
-        result = call_devpi_in_dir(tmpdir, ["devpi-init"])
+    result = call_devpi_in_dir(tmpdir, ["devpi-init"])
     assert os.listdir(tmpdir.strpath) == [".nodeinfo"]
     result.stderr.fnmatch_lines("*already contains devpi-server data*")
 
@@ -352,30 +333,56 @@ def test_serve_max_body(monkeypatch, tmpdir):
     main(["devpi-server", "--max-request-body-size", "42"])
 
 
-def test_root_passwd_option(makexom):
+def test_root_passwd_option(gentmp, makexom, storage_info):
+    from devpi_server.init import init
     # by default the password is empty
-    xom = makexom()
+    serverdir = gentmp()
+    argv = ["devpi-init", "--serverdir", serverdir]
+    if storage_info["name"] != "sqlite":
+        argv.append("--storage=%s" % storage_info["name"])
+    init(argv=argv)
+    xom = makexom(opts=("--serverdir", serverdir))
     with xom.keyfs.transaction(write=False):
         user = xom.model.get_user('root')
         assert user.validate("")
         assert not user.validate("foobar")
     # the password can be set from the command line
-    xom = makexom(["--root-passwd", "foobar"])
+    serverdir = gentmp()
+    argv = [
+        "devpi-init", "--serverdir", serverdir,
+        "--root-passwd", "foobar"]
+    if storage_info["name"] != "sqlite":
+        argv.append("--storage=%s" % storage_info["name"])
+    init(argv=argv)
+    xom = makexom(opts=("--serverdir", serverdir))
     with xom.keyfs.transaction(write=False):
         user = xom.model.get_user('root')
         assert not user.validate("")
         assert user.validate("foobar")
 
 
-def test_root_passwd_hash_option(makexom):
+def test_root_passwd_hash_option(gentmp, makexom, storage_info):
+    from devpi_server.init import init
     # by default the password is empty
-    xom = makexom()
+    serverdir = gentmp()
+    argv = ["devpi-init", "--serverdir", serverdir]
+    if storage_info["name"] != "sqlite":
+        argv.append("--storage=%s" % storage_info["name"])
+    init(argv=argv)
+    xom = makexom(opts=("--serverdir", serverdir))
     with xom.keyfs.transaction(write=False):
         user = xom.model.get_user('root')
         assert user.validate("")
         assert not user.validate("foobar")
     # the password hash can be directly set from the command line
-    xom = makexom(["--root-passwd-hash", "$argon2i$v=19$m=102400,t=2,p=8$j9G6V8o5B0Co9f4fQ6gVIg$WzcG2C5Bv0LwtzPWeBcz0g"])
+    serverdir = gentmp()
+    argv = [
+        "devpi-init", "--serverdir", serverdir,
+        "--root-passwd-hash", "$argon2i$v=19$m=102400,t=2,p=8$j9G6V8o5B0Co9f4fQ6gVIg$WzcG2C5Bv0LwtzPWeBcz0g"]
+    if storage_info["name"] != "sqlite":
+        argv.append("--storage=%s" % storage_info["name"])
+    init(argv=argv)
+    xom = makexom(opts=("--serverdir", serverdir))
     # we allow writes, because the hash might be upgraded
     # this happend after passlib 1.7.2 made argon2id the default
     with xom.keyfs.transaction(write=True):
